<?php

namespace Agi\Action;
use \IvozProvider\Model\Features;

/**
 * @class ExternalFaxCallAction
 *
 * @brief Manage outgoing external calls generated by faxes
 *
 */

class ExternalFaxCallAction extends ExternalCallAction
{
    protected $_number;

    protected $_faxfile;

    public function setFaxfile($faxfile)
    {
        $this->_faxfile = $faxfile;
        return $this;
    }

    public function setDestination($number)
    {
        $this->_number = $number;
        return $this;
    }

    public function process()
    {
        // Local variables
        $fax = $this->agi->getChannelCaller();
        $faxfile = $this->_faxfile;
        $number = $this->_number;

        // Get company from the caller
        $company = $fax->getCompany();

        // Some feedback for asterisk cli
        $this->agi->notice("Processing External call from fax \e[0;32m%s [fax%d]\e[0;93m to %s",
            $fax->getName(), $fax->getId(), $number);

        // Convert to E.164 format
        $e164number = $company->preferredToE164($number);

        // Check if outgoing call can be tarificated
        if (!$this->checkTarificable($e164number)) {
            $this->agi->error("Destination %s can not be billed.", $e164number);
            $this->agi->decline();
            $faxfile->setStatus('error')->save();
            return;
        }

        // Get Ougoing presentation
        $ddi = $fax->getOutgoingDDI();

        // We need Outgoing DDI for external call presentation
        if (!$ddi) {
            $this->agi->error("Fax %s [fax%d] has not OutgoingDDI configured", $fax->getName(), $fax->getId());
            $this->agi->decline();
            $faxfile->setStatus('error')->save();
            return;
        }

        // Check if DDI belong to platform
        $this->checkDDIBounced($e164number);

        // Set Caller name
        $this->agi->setCallerIDNum($ddi->getDDIE164());

        // Call the PSJIP endpoint
        $this->agi->setVariable("__DIAL_DST", "PJSIP/" . $e164number . '@proxytrunks');
        $this->agi->setVariable("__DIAL_OPTS", "");
        $this->agi->redirect('faxes-call-world', $e164number);
    }
}
